"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
var React = __importStar(require("react"));
var themed_settings_1 = require("@uifabricshared/themed-settings");
var theming_ramp_1 = require("@uifabricshared/theming-ramp");
var foundation_tokens_1 = require("@uifabricshared/foundation-tokens");
var theming_react_native_1 = require("@uifabricshared/theming-react-native");
/* tslint:disable-next-line no-any */
function getOptionsFromObj(obj) {
    var objType = obj && typeof obj;
    return ((objType === 'object' || objType === 'function') && obj.__composable) || undefined;
}
exports.getOptionsFromObj = getOptionsFromObj;
function _buildGetComponentCache() {
    var weakMap = new WeakMap();
    var nullTheme = {};
    return function (theme) {
        theme = theme || nullTheme;
        return weakMap.get(theme) || weakMap.set(theme, {}).get(theme);
    };
}
function _getSettingsFromTheme(theme, name) {
    return theming_ramp_1.getSettings(theme, name);
}
function _getHasToken(slots) {
    var slotTokens = {};
    Object.keys(slots).forEach(function (slotName) {
        var slot = slots[slotName];
        var slotType = (typeof slot !== 'object' ? slot : slot.slotType);
        var options = getOptionsFromObj(slotType);
        slotTokens[slotName] = (options && options.resolvedTokens && options.resolvedTokens.tokenKeys) || undefined;
    });
    return function (target, key) {
        return slotTokens[target] && slotTokens[target].hasOwnProperty(key);
    };
}
function useStylingCore(props, options, baseKey, getComponentCache, lookupOverride) {
    // get the theme value from the context (or the default theme if it is not set)
    var theme = React.useContext(theming_react_native_1.ThemeContext) || theming_react_native_1.getTheme();
    // get the cache for this component from the theme
    var cache = getComponentCache(theme);
    // resolve the array of settings for these options
    lookupOverride = lookupOverride || props;
    var _a = themed_settings_1.getThemedSettings(options.settings, theme, cache, baseKey, lookupOverride, _getSettingsFromTheme), settings = _a.settings, key = _a.key;
    // finish by processing the tokens and turning IComponentSettings into ISlotProps (this removes things like _overrides)
    return theming_ramp_1.returnAsSlotProps(foundation_tokens_1.processTokens(props, theme, settings, options.resolvedTokens, key, cache));
}
/**
 * return a useStyling implementation, in the form of IUseComposeStyling, based on the passed in styleSettings.  The
 * styleSettings will be captured in the created closure and will be set up to enable the appropriate levels of caching.
 *
 * @param options - style settings to configure this function.  Note that this should be scoped to a single component.
 * @param name - optional base name to use as a cache key
 */
function initializeStyling(options, name) {
    // process the tokens and get them ready to render
    var styles = options.styles, slots = options.slots;
    options.resolvedTokens = foundation_tokens_1.buildComponentTokens(styles, _getHasToken(slots));
    // create a useStyling implementation for this component type (per type, not per instance)
    return function (props, lookupOverride) {
        return useStylingCore(props, options, name, _buildGetComponentCache(), lookupOverride);
    };
}
exports.initializeStyling = initializeStyling;
//# sourceMappingURL=useStyling.js.map